// ivao.cxx - Network interface program to send data to IVAO pilot client
//
// Created by: 	J. Wojnaroski  -- castle@mminternet.com
// Date:		31 AUG 2007 
//
// Extended from original network code developed by C. Olson
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of the
// License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

#include <simgear/debug/logstream.hxx>
#include <simgear/io/iochannel.hxx>

#include <vector>

#include "ivao.hxx"
#include <FDM/flight.hxx>
#include <Main/globals.hxx>
#include <Main/fg_props.hxx>

SG_USING_STD(vector);

FGIVAO::FGIVAO() : 
	press_node(fgGetNode("/environment/pressure-inhg[0]", true)),
	p_latitude(fgGetNode("/position/latitude-deg", true)),	
	p_longitude(fgGetNode("/position/longitude-deg", true)),
	p_elev_node(fgGetNode("/position/altitude-ft", true)),
	p_pitch(fgGetNode("/orientation/pitch-deg", true)),
	p_bank(fgGetNode("/orientation/roll-deg", true)),
	p_heading(fgGetNode("/orientation/heading-deg", true)),
	p_grndspeed(fgGetNode("", true)),
	p_screen(fgGetNode("/sim/startup/fullscreen", true)),
	p_wow(fgGetNode("/gear/gear/wow", true)),
	p_com1(fgGetNode("/instrumentation/comm/frequencies/selected-mhz", true)),
	p_com2(fgGetNode("/instrumentation/comm[1]/frequencies/selected-mhz", true)),
	p_transponder(fgGetNode("/instrumentation/transponder/inputs/id-code", true))
{

}

FGIVAO::~FGIVAO() {
}

// open hailing frequencies
bool FGIVAO::open() {
    if ( is_enabled() ) {
	SG_LOG( SG_IO, SG_ALERT, "This shouldn't happen, but the channel " 
		<< "is already in use, ignoring" );
	return false;
    }

    SGIOChannel *io = get_io_channel();

    if ( ! io->open( get_direction() ) ) {
	SG_LOG( SG_IO, SG_ALERT, "Error opening channel communication layer." );
	return false;
    }

    set_enabled( true );

    return true;
}

void FGIVAO::collect_data( const FGInterface *fdm, ivaoFGData *data ) {
    //data->ivap_signature = "FGIVAP";
		data->version_id = IVAO_VERSION;
		data->screen_state = p_screen->getBoolValue();
    
    data->longitude = p_longitude->getDoubleValue();   
    data->latitude = p_latitude->getDoubleValue();
    data->altitude = p_elev_node->getDoubleValue();

    data->pitch = p_pitch->getDoubleValue();
    data->bank = p_bank->getDoubleValue();
    data->heading = p_heading->getDoubleValue();
    data->groundspeed = cur_fdm_state->get_V_ground_speed();   
		data->wow_main = p_wow->getBoolValue();
		data->static_pressure = press_node->getDoubleValue();
		
		data->com_1 = p_com1->getDoubleValue();
		data->com_2 = p_com2->getDoubleValue();
		data->transponder = p_transponder->getDoubleValue();

//   data->sea_level_pressure = fgGetDouble("/environment/sea-level-pressure-inhg");
}

// process work for this port
bool FGIVAO::process() {
    SGIOChannel *io = get_io_channel();
    int length = sizeof(buf);

  if ( get_direction() == SG_IO_OUT ) {
        collect_data( cur_fdm_state, &buf );
	if ( ! io->write( (char *)(& buf), length ) ) {
	    SG_LOG( SG_IO, SG_ALERT, "Error writing data." );
	    return false;
	}
    } else if ( get_direction() == SG_IO_IN ) {
	if ( io->get_type() == sgFileType ) {
	    if ( io->read( (char *)(& buf), length ) == length ) {
		SG_LOG( SG_IO, SG_DEBUG, "Success reading data." );
//		distribute_data( &buf, cur_fdm_state );
	    }
	} else {
	    while ( io->read( (char *)(& buf), length ) == length ) {
		SG_LOG( SG_IO, SG_DEBUG, "Success reading data." );
//		distribute_data( &buf, cur_fdm_state );
	    }
	}
    }

    return true;
}


// close the channel
bool FGIVAO::close() {
    SGIOChannel *io = get_io_channel();

    set_enabled( false );

    if ( ! io->close() ) {
	return false;
    }

    return true;
}

